// Code generated by mkconsts.go. DO NOT EDIT.

//go:build !goexperiment.regabiwrappers
// +build !goexperiment.regabiwrappers

package goexperiment

const RegabiWrappers = false
const RegabiWrappersInt = 0
