// compiledir

// Copyright 2019 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 33020: gollvm assert in Llvm_backend::materializeComposite

package ignored
