// compiledir

// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 15470: Make sure special-case signatures can
// be exported and imported w/o problems.

package ignored
