// compile

// Copyright 2022 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package p

type T struct{}

func (T) m() []T { return nil }

func f(x T) {
	for _, x := range func() []T {
		return x.m()
	}() {
		_ = x
	}
}
