// Copyright 2011 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file

package one

func Foo() (n int64, _ *int) {
	return 42, nil
}

