// Copyright 2022 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package a

type A[T any] struct{}

func (_ A[T]) Method() {}

func DoSomething[P any]() {
	a := A[*byte]{}
	a.Method()
}
